import { SelectQueryBuilder } from "typeorm";
/**
 * Pagination utility.
 *
 * @author Jeongho Nam - https://github.com/samchon
 */
export declare namespace Paginator {
    /**
     * A page.
     *
     * Collection of records with pagination indformation.
     */
    interface IPage<T extends object> {
        /**
         * Page information.
         */
        pagination: IPagination;
        /**
         * List of records.
         */
        data: T[];
    }
    /**
     * Page information.
     */
    interface IPagination {
        /**
         * Current page number.
         */
        page: number;
        /**
         * Limitation of records per a page.
         *
         * @default 100
         */
        limit: number;
        /**
         * Total records in the database.
         */
        total_count: number;
        /**
         * Total pages.
         *
         * Equal to {@link total_count} / {@link limit} with ceiling.
         */
        total_pages: number;
    }
    /**
     * Page request data
     */
    interface IRequest {
        /**
         * Page number.
         */
        page?: number;
        /**
         * Limitation of records per a page.
         */
        limit?: number;
    }
    /**
     * Type of transformer.
     *
     * @param records Records to be transformed
     * @returns Transformed records
     */
    type Transformer<Input extends object, Output extends object> = (records: Input[]) => Output[] | Promise<Output[]>;
    /**
     * Pagination from regular ORM entity instance.
     *
     * Paginate regular ORM entity instances by getting from
     * {@link SelectQueryBuilder.getMany} method and transforming them to *Output* type.
     *
     * @template Entity Target entity type
     * @template Output Output data type
     * @param stmt Select query builder for Entity type
     * @param request Page request info
     * @param transformer Transformer function coverting Entity to Output
     * @returns Paginated Output data
     */
    function regular<Entity extends object, Output extends object>(stmt: SelectQueryBuilder<Entity>, request: IRequest, transformer: Transformer<Entity, Output>): Promise<IPage<Output>>;
    /**
     * Pagination from raw query data.
     *
     * Paginate raw data by getting from {@link SelectQueryBuilder.getRawMany} method.
     * If required, transform the raw data to *Output* type.
     *
     * @template Entity Target entity type
     * @template Output Output data type
     * @param stmt Select query builder for Entity type
     * @param request Page request info
     * @param transformer Transformer function coverting raw data to Output, if required
     * @returns Paginated Output data
     */
    function raw<Entity extends object, Output extends object>(stmt: SelectQueryBuilder<Entity>, request: IRequest, transformer?: Transformer<any, Output>): Promise<IPage<Output>>;
    function orderBy<T, Field extends string, Sortable extends Array<`-${Field}` | `+${Field}`>>(stmt: SelectQueryBuilder<T>, fieldList: Sortable, dictionary?: Record<Field, string | (() => string)>, groupped?: boolean): void;
    /**
     * Follow order of source.
     *
     * Sort target array by following the order of source array.
     *
     * @param target Target array to sort
     * @param source Source array to be referenced
     * @param keyGetter Key getter function from any record
     * @return Target array itself
     */
    function follow<Target extends {
        id: Type;
    }, Source extends {
        id: Type;
    }, Type extends string | number>(target: Target[], source: Source[], keyGetter?: (elem: Target) => string | number | null): Target[];
    const DEFAULT_LIMIT = 100;
}
